#This code has been prepared to support the simulations presented in the paper "Financing drug discovery for orphan diseases" 
#by D. Fagnan, A. Gromatzky, R. Stein, JM. Fernandez, A. Lo published in Drug Discovery Today in Nov 2013. 
#Input:
#@show_progress is of type boolean and if true displays a status bar of the current simulation
#@params is of type struct and is parameter object which is the output of the init_params_fn function (optional)
#Output:
#@results is of type struct and provides details of each simulation run
module DrugSimulator
    
using UtilityFunctions
using Distributions,DistributionsExtended
using DrugParams
import GeneralCorrelation.initSuccessProbs ##Currently testing used for arbitrary scientific correlation

export simulate_CFs_fn


function simulate_CFs_fn(params,show_progress=true,debug=false)

    #Nested Functions
    #Changes guarantee_draw,drew_on_guarantee
    function repay_guarantee_ffn(cur_per,cur_cash,cur_guarantee)
        #Experimental function, not currently used.
        ce_pmt_due = ce.ce_amt-cur_guarantee
        ce_pmt = min(cur_cash,ce_pmt_due)
        cur_cash -= ce_pmt
        cur_guarantee += ce_pmt
        return cur_cash,cur_guarantee
    end

    #Nested Function
    #Changes guarantee_draw,drew_on_guarantee
    function draw_from_guarantee_ffn(cur_per,cur_cash,cur_guarantee)
        draw_amt = cur_cash
        guarantee_draw[nsimu,cur_per] += draw_amt
        cur_cash = 0
        cur_guarantee += draw_amt
        if drew_on_guarantee[nsimu] == 0
            drew_on_guarantee[nsimu] = cur_per 
        end
        return cur_cash,cur_guarantee
    end

    #Nested Function
    #Accesses params
    function  calc_svc_due_this_per_ffn(values)    
        #Use of ce.premium is experimental and untested.
        svc_due = ce.ce_amt*ce.ce_premium
        for j = 1:length(values)
            svc_due+=bonds.servicing_rate*values[j]
        end
        return svc_due
    end

    #Nested function
    #Accesses params,bond_value
    #Changes A1_in_default, A2_in_default
    function check_for_default_ffn(cur_per,cur_cash,cur_guarantee)
        svc_due  = calc_svc_due_this_per_ffn(bond_value) + unpaid_svc*(1+bonds.svc_accrual_int_rate)
        A1_cash_required = AMORT_SCHED[1,cur_per]+bond_value[1]*bonds.coupon[1]
        A2_cash_required = AMORT_SCHED[2,cur_per]+bond_value[2]*bonds.coupon[2]
        if cur_cash+cur_guarantee<svc_due
            svc_defaults[nsimu] = true
            if bond_value[1]>0 
                A1_defaults[nsimu] = true
            end
            if bond_value[2]>0
                A2_defaults[nsimu] = true
            end
        elseif cur_cash+cur_guarantee<svc_due+A1_cash_required
            A1_defaults[nsimu] = true
            if bond_value[2]>0
                A2_defaults[nsimu] = true
            end
        elseif (cur_cash+cur_guarantee<svc_due+A1_cash_required+A2_cash_required) && (bond_value[2]>0)
            A2_defaults[nsimu] = true
        end
        if debug > 1
            println("svc-default: $(svc_defaults[nsimu]) A1-default: $(A1_defaults[nsimu]) A2-default: $(A2_defaults[nsimu])")
        end
        return svc_defaults[nsimu] || A1_defaults[nsimu] || A2_defaults[nsimu]
        
    end

    #Nested function
    #Changes interest_paid, drew_on_guarantee, guarantee_draw
    #Accesses params
    function pay_principal_ffn(cur_per,cur_cash,cur_guarantee)
       #PAY P&I
       for b = 1:NBONDS #pay principal
           P = AMORT_SCHED[b,cur_per]
           principal_paid[b,cur_per] = min(P,cur_cash+cur_guarantee)
           cur_cash        = cur_cash-principal_paid[b,cur_per]
           bond_value[b]       = bond_value[b]-principal_paid[b,cur_per]
           if bond_value[b]<0.0001
               bond_value[b]   = 0
           end
       end
       return cur_cash
       
    end

    #Nested function
    #Changes interest_paid, drew_on_guarantee, guarantee_draw
    #Accesses params
    function pay_interest_ffn(cur_per,cur_cash,cur_guarantee)
        for b = 1:NBONDS # pay int
            I = bonds.coupon[b]*bond_value[b]
            interest_paid[b,cur_per] = min(I,cur_cash+cur_guarantee) 
            cur_cash       = cur_cash-interest_paid[b,cur_per]
        end          
        return cur_cash
    end

    #Nested function
    #Changes unpaid_svc
    #Accesses params, in_default, bond_value, unpaid_svc, history_service, unpaid_svc
    function pay_servicing_ffn(cur_per,cur_cash,cur_guarantee)
        svc_due_this_per  = calc_svc_due_this_per_ffn(bond_value)
        svc_past_due      = unpaid_svc*(1+bonds.svc_accrual_int_rate)
        svc_due           = svc_due_this_per + svc_past_due
        svc_paid          = min(svc_due,cur_cash+cur_guarantee)

        #update main function variables
        history_service[nsimu] =  history_service[nsimu] + svc_paid
        unpaid_svc         =  round((svc_due-svc_paid)*10^5)/10^5
        cur_cash           =  cur_cash-svc_paid
        return cur_cash 
    end

    #Nested function
    #Accesses NPERS, unpaid_svc, bond_value, params, AMORT_SCHED
    function ic_test_result_ffn(b, cur_per, cur_cash)
        if cur_per !=NPERS
            sched_svc = unpaid_svc
            bv = bond_value[b]
            K  = cur_cash
            #For bonds after the first, must subtract cash paid for previous
            #bonds.
            if b>1
                for bb = 1:(b-1)
                    btemp = bond_value[bb]
                    N = min(bonds.IC_pers,NPERS-cur_per)
                    for m = 1:N
                        K = K-AMORT_SCHED[bb,cur_per+m]-bonds.coupon[bb]*btemp-bonds.servicing_rate*btemp
                        btemp = btemp-AMORT_SCHED[bb,cur_per+m]
                    end
                end
            end
            sched_P  =  0.0
            sched_I  =  0.0
           
            for m = 1:min(bonds.IC_pers,NPERS-cur_per)
                sched_P =  sched_P + AMORT_SCHED[b,cur_per+m]
                sched_I =  sched_I + bonds.coupon[b]*bv
                sched_svc = sched_svc+bonds.servicing_rate*bv
                bv = bv-AMORT_SCHED[b,cur_per+m]        
            end
            cash_due = sched_P+sched_I+sched_svc
            ratio = K/cash_due
        else
            ratio = bonds.interest_coverage[b]
            cash_due = 0.0
            K = cur_cash
        end
        return ratio, cash_due, K
    end

    #Nested function
    #Changes compounds,cash,sales,sale_times, sell_value or extra_cash_from_coverage,time_in_phase
    function update_sale(index,price,cur_per,nsold,forCoverage,forLiquidation=false)
        #index may be logical or indices 
        cassets=assets[COMPOUND_TYPES[index]]
        sale_per = min(cur_per+iceil(cassets.sale_time[compounds[index]]),HORIZON)
		if forCoverage || forLiquidation
			sale_per += iceil(cassets.sale_delay[compounds[index]]-time_since_target[index])
			sale_per = min(sale_per,HORIZON)
		end
        len = length(price) 
		len !=1 && error("Should not occur - multiple sales at once.")

        sell_value[index]=price


        for i=1:len
            sales[nsimu,compounds[index][i]] += 1
			##modify sale time to be for period cash cleared.
        	sale_times[nsimu,sale_per[i]]+= 1
            if forCoverage
                extra_cash_from_coverage[nsimu,sale_per[i]] += price[i] 
            end
            ##Payout reserve to structured equity only if debt is paid off.
            #println("$nsimu reserve=$reserve_flag, comp==$(compounds[index][i]) sell==$(sell_phase[index][i])")
            
            if reserve_flag & (compounds[index][i]==sell_phase[index][i]) & (!forCoverage)
				pay_reserve(price[i],sale_per[i],nsold)
            else
	        	cash[sale_per[i]]+=price[i] 
            end
        end
        compounds[index]    = cassets.trans_prob_col.SLD 
        time_in_phase[index]= 0 
    end
	function pay_reserve(price,sale_per,nsold)
		#println("$nsimu num sold = $nsold_prev")

        reserveprice = price*sum(tranche_weights[nsold,:])

        if reserveprice < price
##If not all of the compound is being paid out
	        res_slice = slice(reserve_cash,nsimu,sale_per,:)
            res_slice[:]+=reserveprice*slice(tranche_weights,nsold,:)
            reserve[nsimu,nsold]+=price ##only used for bookmaring!
            cash[sale_per]+=price-reserveprice
        else
    	    res_slice = slice(reserve_cash,nsimu,sale_per,:)
            res_slice[:]+=price*slice(tranche_weights,nsold,:)
            reserve[nsimu,nsold]+=price ##only used for bookmaring!
        end
       		##adjust for capped piece
        for eq = 1:length(params.simu.tranche_capratio)
        	totalPaid = sum(reserve_cash[nsimu,:,eq])
            capPaid = params.simu.tranche_capratio[eq]*params.simu.initial_cash*params.simu.tranche_cashweights[eq]
            if totalPaid > capPaid
            	reserve_cash[nsimu,sale_per,eq]-=(totalPaid-capPaid)
                ##currently not fixed for last equity piece.
                reserve_cash[nsimu,sale_per,eq+1]+=(totalPaid-capPaid)
            end
        end
 
	end	
    function update_milestones(index,cur_per,nsold)
        #index may be logical or indices 
        cassets = assets[COMPOUND_TYPES[index]]
        milestone=cassets.strucExit.milestone
        if transitioned[index] || !invested[index] || isempty(cassets.midValueDist)
        else
            milestone=cassets.strucExit.milestoneMid
        end
        sold_phase = compounds[index]
        time = cur_per+iceil(cassets.sale_time[sold_phase])
        while (sold_phase != cassets.trans_prob_col.DSC) && (sold_phase != cassets.trans_prob_col.APP)
            time += iceil(time_required[index,compounds[index]])-time_in_phase[index]
			time = int(time)
            time_in_phase[index]=ifloor(time_required[index,compounds[index]])
            perform_transitions_ffn(index,time_required[index,compounds[index]],success[index,compounds[index]])
            if compounds[index]==cassets.trans_prob_col.DSC
                break
            end
            if isdefined(milestone,sold_phase,compounds[index])
                dist = milestone[sold_phase,compounds[index]]
                price = get_price(index,dist)
                if time > HORIZON 
                    if !warn_flag
                        warn("Milestone occurred at $time which is passed $HORIZON")
                        warn_flag = true
                    end
				elseif reserve_flag 
					pay_reserve(price,time,nsold)
                elseif (time > LIQHORIZON) && (time > simu.MINPAYOFFTIME)     
					payoff_slice = slice(payoff,nsimu,time,:)
					payoff_slice[:] += price*params.simu.tranche_cashweights
                    cash_from_milestones[nsimu,time] += price
                else
                    cash[time] += price
                    cash_from_milestones[nsimu,time] += price
                end
                
            end
            time_in_phase[index]=0
            if compounds[index]==cassets.trans_prob_col.APP
                break
            end
        end
        compounds[index]=sold_phase ##set compound back to starting phase.
    end
   
    #Nested Function
    function get_price(rem_index::Int64,dist,mx=Inf)
        cassets=assets[COMPOUND_TYPES[rem_index]]
        
        price   = compound_sale_price_fn(dist, mx, rho, CORR_DRAW)*assets[COMPOUND_TYPES[rem_index]].equity_stake
        # here is where we adjust for uninvested compounds
        ## incompatible with old code unless midValue is null
        if !invested[rem_index] && isempty(cassets.midValueDist)  
            price = max(price-invest_cost[rem_index],0)
        end
        #adjust for compounds that have not transitioned at all until liquidation
        if time_in_phase[rem_index]>=NPERS-1
            price*=cassets.ratio_unchanged
        end
        ##adjust for performance fee e.g. 20%

        if !isnan(simu.performance_fee)
           price = price - (simu.performance_fee)*max(price-total_invest_cost[rem_index],0)
        end
        return price
    end
    #Nested Function
    #Accesses params, compounds, HORIZON, equity_in_compound, invested, invest_cost, s, cash, time_in_phase, extra_cash_from_coverage, sales, sale_times
    function sell_compounds_to_cover_shortfall_ffn(remaining_compounds,shortfall,cur_per)
        #  We simply sell the most mature compounds first
        len = length(remaining_compounds)
        if len <= 0
            return remaining_compounds
        end
        price = fill(NaN,NCOMPOUNDS)
        for j in remaining_compounds 
             cassets=assets[COMPOUND_TYPES[j]]
             if transitioned[j] || !invested[j] || isempty(cassets.midValueDist)
                dist = cassets.valueDist[compounds[j]]
            else
                dist = cassets.midValueDist[compounds[j]]
            end
            price[j] = get_price(j,dist)
        end
        #Order in which to buy the compounds, sort by highest cost?
        sorted_idx  = reverse!(sortperm(price))
        amount_sold = 0
        for j in sorted_idx
            if isnan(price[j]) 
                continue    
            end
            amount_sold+=price[j]
			
        	nsold_prev = sum(reserve[nsimu,:].>0)
            update_sale(j,price[j],cur_per,nsold_prev+1,true)    
            delete!(remaining_compounds,j)  
            if amount_sold > shortfall
                break
            end
        end
        return remaining_compounds
    end

    #Nested Function
    #Accesses NCOMPOUNDS, compounds, params, HORIZON, rho, z, equity_in_compound, invested, invest_cost, time_in_phase, sell_value, sales, sale_times, cash
    function liquidate_portfolio_ffn(remaining_compounds,cur_per)
        sold_index = IntSet()
        for j in remaining_compounds
            sell_compound_ffn(j,cur_per,true)
            push!(sold_index,j)
        end
        setdiff!(remaining_compounds,sold_index)
        return remaining_compounds
    end

    #This needs to be a nested-function within simulate_CFs_fn
    #Accesses 
    #compounds, invested, params, sales, sale_times, cash, time_in_phase, HORIZON, equity_in_compound, invest_cost, sell_value, s, z
    function sell_compound_ffn(comp_idx, cur_per,forLiquidation=false)
        nsold_prev = sum(reserve[nsimu,:].>0)
        cassets=assets[COMPOUND_TYPES[comp_idx]]
        if rand() < cassets.probUpfrontExit[compounds[comp_idx]]
            if transitioned[comp_idx] || !invested[comp_idx] || isempty(cassets.midValueDist)
                dist = cassets.valueDist[compounds[comp_idx]]
            else
                dist = cassets.midValueDist[compounds[comp_idx]]
            end
        else
            cassets=cassets.strucExit
            if transitioned[comp_idx] || !invested[comp_idx] || isempty(cassets.midValueDist)
                dist = cassets.valueDist[compounds[comp_idx]]
            else
                dist = cassets.midValueDist[compounds[comp_idx]]
            end
            update_milestones(comp_idx,cur_per,nsold_prev+1) ##UPDATE DELAYED MILESTONES
        end

        price = get_price(comp_idx,dist)  ###SIMPLE SALE
        if debug > 1
            println("Selling $comp_idx in phase $(compounds[comp_idx]) for $price.")
        end
        update_sale(comp_idx,price,cur_per,nsold_prev+1,false,forLiquidation)
    end

    function perform_transitions_ffn(index::Int64,time_required,success)
        p=rand()
        cum = 0;
        if debug > 1
            print("Transitioning $index from $(compounds[index]) to")
        end
        cassets=assets[COMPOUND_TYPES[index]]
        if cassets.timeMode=="Markov"
            dist = cassets.trans_prob[compounds[index],:]
            cdist = cumsum(dist,2);
            len   = size(cdist,2);
            cdist[:,len] = 1;
            compounds[index] = findfirst(cdist.>=p)
        else
            if time_in_phase[index]+1 >= time_required
                if p < success
                    compounds[index]+=1
                else
                    compounds[index]=1
                end     
            end
        end
        if debug > 1
            println(compounds[index])
            println("Time required: $time_required, time spent: $(time_in_phase[index])")
        end
    end
    function reset_variables_ffn()
        # state of compounds
        for k = 1:NCOMPOUNDS
            invested[k] = true
            time_in_phase[k] = 0
            time_since_target[k]=0
            sell_value[k] = 0
            compounds[k]=STARTING_STATES[k]
            invest_cost[k]  = trial_cost_fn(assets[COMPOUND_TYPES[k]].costDist[compounds[k]],Inf)
            #money_spent[k] = 0 
            #money_saved[k] = 0 
        end
        for k = 1:HORIZON
            cash[k] = 0
            guarantee[k] = 0
        end
        for k = 1:NBONDS, l =1:NPERS
            ic_ratio[k,l]=0
            ic_shortfall[k,l]=0
        end
        for k = 1:NBONDS+1
            bond_value[k]=bonds.nominal[k];
        end

        #record keeping 
        interest_paid   = zeros(NBONDS, NPERS)
        principal_paid  = zeros(NBONDS, NPERS)
        
        cash[1]         = first_cash
        cash_begin_per[nsimu,1] = cash[1]
           
        guarantee[1]    = first_guarantee
        guarantee_begin_per[nsimu,1] = guarantee[1]

        tranche_weights      = copy(simu.tranche_weights)
        tranche_cashweights  = copy(simu.tranche_cashweights)
    
        unpaid_svc      = 0.0
        in_default      = false
    end
    function buy_initial_compounds_ffn(cash_left_to_spend)
        compounds_remaining = IntSet()
        for ctr = 1:NCOMPOUNDS
            if STARTING_PERIOD[ctr]==1
                cassets=assets[COMPOUND_TYPES[ctr]]
                price_tmp = invest_cost[ctr] + cassets.upfront[compounds[ctr]]
                future_cost_est = sell_phase[ctr]<=0 ? 0 : cassets.budget[compounds[ctr]]
                if cash_left_to_spend   >= price_tmp + future_cost_est 
                    money_spent[ctr]     =  price_tmp
                    money_saved[ctr]     =  future_cost_est 
                    cash_left_to_spend  -= price_tmp + future_cost_est 
                    cash[1]             -= price_tmp
                    compounds_bought[nsimu,1,compounds[ctr]] +=  1
                    push!(compounds_remaining,ctr)
                    if debug > 1
                        println("buying: $ctr\t for $price_tmp, saving $future_cost_est with cash left: $cash_left_to_spend")
                    end
                else 
                    compounds[ctr] = cassets.trans_prob_col.DSC
                end
            end
        end 
        return compounds_remaining
    end
            
    #Nested function
    #Accesses compounds
    #Changes  invested, invest_cost, time_in_phase
    function transition_compound_ffn(idx::Int64)
        cassets=assets[COMPOUND_TYPES[idx]]
        if compounds[idx] == cassets.trans_prob_col.NDA
            invested[idx] = true
        else
            invested[idx]   = false
            invest_cost[idx]  = trial_cost_fn(cassets.costDist[compounds[idx]],Inf)+cassets.milestone[compounds[idx]]
        end
        time_in_phase[idx]= 0
    end

	function getSellPhase(sell_phase::Array{Int64,1},compounds::Array{Int64,1})
	    sell_phase[compounds]
	end

	function getSellPhase(sell_phase::Array{Float64,1},compounds::Array{Int64,1})
	    rand(Categorical(sell_phase),size(compounds))
	end


    if show_progress
		println("Starting simulation...")
        prog_bar_click = div(params.simu.NSIMUS,10)
    end

    #setup global variables and constants
    const simu                  = params.simu
    const assets                = params.assets
    const bonds                 = params.bonds
	const ce                    = params.ce
   	const NSIMUS                = simu.NSIMUS
	const reserve_flag          = simu.reserve_flag
   	const NPERS                 = simu.TIMESTEPS
    const STEPS_YR              = simu.STEPS_YR
	const NEQUITY               = size(simu.tranche_weights,2)
    ##TODO: FIX ASSETS TYPE WITH DIFFERENT SALE TIMES
    const LIQHORIZON            = NPERS+int(ceil(maximum(assets[1].sale_time+assets[1].sale_delay)))
    const HORIZON               = 4*LIQHORIZON
   	const NCOMPOUNDS            = int(sum(vec(simu.initial_compounds))) 
	const NSTATES               = size(simu.initial_compounds,2) 
	const NBONDS                = length(bonds.capital_structure)-1
    const NTYPES                = length(params.assets)
    const COMPOUND_TYPES        = zeros(Integer,NCOMPOUNDS)
	const STARTING_STATES       = zeros(Integer,NCOMPOUNDS)
	const STARTING_PERIOD       = zeros(Integer,NCOMPOUNDS) 
	const AMORT_SCHED           = zeros(eltype(bonds.nominal),NBONDS,NPERS)
	const sell_phase            = zeros(Integer,NCOMPOUNDS)
    const warn_flag = false
    
	temp = 1
	for period=1:NPERS
		for state = 1:NSTATES
            for compoundtype = 1:NTYPES
    			if simu.initial_compounds[period,state,compoundtype]>0
                    ncomp = simu.initial_compounds[period,state,compoundtype]
    				STARTING_STATES[temp:temp+ncomp-1]  = state
    				STARTING_PERIOD[temp:temp+ncomp-1]  = period
                    COMPOUND_TYPES[temp:temp+ncomp-1]   = compoundtype
                    temp+=ncomp
    			end
            end
		end
	end

	temp_bond_value = copy(bonds.nominal)
	for cctr = 1:NPERS
		for rctr = 1:NBONDS
			AMORT_SCHED[rctr,cctr] = calc_amort_pmt_due_fn(cctr, 
            	bonds.amort_timing[rctr,1], 
            	bonds.amort_timing[rctr,2],
            	bonds.nominal[rctr], temp_bond_value[rctr])
			temp_bond_value[rctr]  -= AMORT_SCHED[rctr,cctr]
		end
    end
    
    # ----------------------------------
	const rho                 = assets[1].rho
	const bond_value          = copy(bonds.nominal)
	const invested            = trues(NCOMPOUNDS)
    const transitioned        = falses(NCOMPOUNDS)
	const invest_cost         = zeros(NCOMPOUNDS)
    const total_invest_cost   = zeros(NCOMPOUNDS)
	const sell_value          = zeros(NCOMPOUNDS)
	const compounds           = zeros(Int64,NCOMPOUNDS)
	const time_in_phase       = zeros(NCOMPOUNDS)
    const time_since_target   = zeros(NCOMPOUNDS)
    const success             = zeros(NCOMPOUNDS, NSTATES)
	
    const first_guarantee     = ce.ce_amt
	const first_cash          = simu.initial_cash
	
	# state of securities  
	const IC_ratio            = zeros(NBONDS, NPERS)
	
	#record keeping 
	const sales     		  = zeros(NSIMUS, NSTATES) 
	const withdrawals         = zeros(NSIMUS, NSTATES)
	const sale_times          = zeros(NSIMUS, LIQHORIZON)
	const withdrawal_times    = zeros(NSIMUS, LIQHORIZON)
	const compounds_to_fund   = zeros(NSIMUS, LIQHORIZON)
	const compounds_funded    = zeros(NSIMUS, LIQHORIZON)
    const cash_from_milestones= zeros(NSIMUS,HORIZON)
    const extra_cash_from_coverage  = zeros(NSIMUS,HORIZON) 
    const cash_realized       = zeros(NSIMUS,HORIZON)
    const cash_begin_per      = zeros(NSIMUS,HORIZON)    
    const guarantee_begin_per = zeros(NSIMUS,HORIZON)
    
    const reserve_cash        = zeros(NSIMUS,HORIZON,NEQUITY)
    const payoff              = zeros(NSIMUS,HORIZON,NEQUITY)

	const compounds_bought    = zeros(NSIMUS, HORIZON, NSTATES)
    const compounds_current   = zeros(NSIMUS, HORIZON, NSTATES)

	const final_compounds     = zeros(NSIMUS, NSTATES)
	const guarantee_draw      = zeros(NSIMUS, HORIZON)
    const drew_on_guarantee   = zeros(NSIMUS)
    
	const cash_for_investment         = zeros(NSIMUS,NPERS)
	const cash_for_buying           = zeros(NSIMUS,NPERS)
	const interest_paid	            = zeros(NBONDS+1, NPERS)
	const principal_paid              = zeros(NBONDS+1, NPERS)
	const history_service		        = zeros(NSIMUS)
	const reserve                   = zeros(NSIMUS,NCOMPOUNDS)
    
	const A1_realized_vals            = zeros(NSIMUS,NPERS)
	const A2_realized_vals            = zeros(NSIMUS,NPERS)
	const A1_paid                     = zeros(NSIMUS,NPERS)
	const A2_paid                     = zeros(NSIMUS,NPERS)
	
	const A1_defaults                 = falses(NSIMUS)
	const A2_defaults                 = falses(NSIMUS)
    const svc_defaults                = falses(NSIMUS)
	const return_on_equity            = zeros(NSIMUS,NEQUITY)
    const irr_on_equity               = zeros(NSIMUS,NEQUITY)
    const weight_tranche             = zeros(NSIMUS,NCOMPOUNDS,NEQUITY)
    const weight_cash                = zeros(NSIMUS,NEQUITY)

	#FIX THESE GLOBAL VARIABLE ISSUES
	unpaid_svc		  = NaN 
	CORR_DRAW 		  = NaN 
	in_default        = false
	const cash 		  = zeros(HORIZON)
    const tranche_weights      = zeros(NCOMPOUNDS,NEQUITY)
    const tranche_cashweights  = zeros(NEQUITY)
    
	const guarantee   = zeros(HORIZON)
	const money_spent = zeros(NCOMPOUNDS)
	const money_saved = zeros(NCOMPOUNDS)
	const ic_ratio    = zeros(NBONDS,NPERS)
	const ic_shortfall= zeros(NBONDS,NPERS)
    const time_required = zeros(NCOMPOUNDS,NSTATES)
	if show_progress 
        println("Simulation of $NSIMUS paths: 0% complete")
    end
	nsimu = 1

    ##Loop over simulation paths.
    for nsimu = 1 : NSIMUS
        ##Print progress bar
		if(show_progress) 
			if (mod(nsimu,prog_bar_click) == 0)
                println("Simulation $(nsimu/NSIMUS*100)% complete")
            end
        end
		
		# reset variables for next path of simulation
	    reset_variables_ffn()	
	        
		# temporary variables to make sure we spend only our target percentage
	    cash_left_to_spend =  cash[1] - bonds.IC_pers*bonds.nominal[1]*bonds.coupon[1] -bonds.IC_pers*bonds.nominal[2]*bonds.coupon[2]
	   	# If we have enough cash, buy it and update our initial cash
       	compounds_remaining = buy_initial_compounds_ffn(cash_left_to_spend) 
      	if debug > 0
			println("Cash remaining: $(cash[1]).")
		end	
		cash[2]             = cash[1] 
		cash_begin_per[nsimu,2] = cash[1]
        guarantee[2]             = guarantee[1]
        guarantee_begin_per[nsimu,2] = guarantee[1]
        #----- simulate one trajectory ----
		#
		# I) For each time period
		#    1) Check for transitions
		#	 2) Apply waterfall rules
		#		A) if insufficient cash to pay this period's obligations transaction is in default
		#			i) liquidate portfolio 
		#			ii) use proceeds to pay servicing and then all P&I for A1, then P&I for A2, etc.
		#		B) if not in default
		#			i) check coverage test 
		#			ii) If coverage is not OK 
		#				a) sell enough assets to bring ratio back in line (coarse approximation)
		#			iii)Pay obligations 
		#				a) Pay servicing
		#				b) Pay P&I for A1, then for A2, etc.
		#				c) make investmensts in compounds as cash permits
       #
		# II) If last time period
		#	1) liquidate portfolio
		#	2) Make final payements to bonds
		#	3) Residual cash goes to equity
		#
		if debug > 0
			println(sell_phase)
		end
		#Valuation for average market component.
		CORR_DRAW = randn()
        SCIENCE_CORR_DRAW = rand()

        time_required = zeros(NCOMPOUNDS,NSTATES)
        mean_time_required = zeros(NCOMPOUNDS,NSTATES)
		success 	  = zeros(NCOMPOUNDS,NSTATES)
		
        for ct in unique(COMPOUND_TYPES)
            ##No steady or mean time information provided.
            assets[ct].timeMode=="Markov" && continue

            mean_success= assets[ct].steady

            if length(mean_success)==(NSTATES-2)
                mean_success = [0,mean_success,0] ##old style parameters that omit absorbing
            end

            tIDX  = COMPOUND_TYPES.==ct
            ncomp = sum(tIDX)
    		for i = 2:NSTATES-1 #skip absorbing states
                if assets[ct].timeMode=="Period"
                    time_required[tIDX,i] = 1+rand(assets[ct].timeDist[i],ncomp)
                    mean_time_required[tIDX,i]=1+mean(assets[ct].timeDist[i])
                else
                    time_required[tIDX,i] = STEPS_YR*rand(assets[ct].timeDist[i],ncomp)
                    mean_time_required[tIDX,i]= STEPS_YR*mean(assets[ct].timeDist[i])
                end
    			success[tIDX,i] = ones(ncomp)*mean_success[i]
    		end #END STATES
        end #COMP TYPES

        if !isempty(simu.successDist) 
            success[:,2:NSTATES-1]=initSuccessProbs(success[:,2:NSTATES-1],simu.rhos,simu.successDist[2:NSTATES-1])
        end

        ## SCIENCE HIT FACTOR
        for ct in unique(COMPOUND_TYPES)
            tIDX  = COMPOUND_TYPES.==ct
            if SCIENCE_CORR_DRAW < assets[ct].science_hitrate
                success[tIDX,:]*=assets[ct].science_hitfactor
            end
        end
        ## RANDOM SELL PHASE
        for ct in unique(COMPOUND_TYPES)
            tIDX  = COMPOUND_TYPES.==ct
            sell_phase[tIDX] = getSellPhase(assets[ct].sell_in_phase,compounds[tIDX])
        end



        
		for i = 2:NPERS # during life of bonds
            for j in compounds_remaining
                if compounds[j] > 0 
                    compounds_current[nsimu,i,compounds[j]]+=1
                end
            end
    
			if debug > 0
				println("Starting period $i")
				if debug >1
					println(compounds)
				end
			end	

            ##REDRAW VALUATION FACTOR IF DESIRED
            if mod(i-1,simu.vCorrPeriods)==0
                CORR_DRAW = randn()
            end
			
            # ----------- first determine current state of portfolio
			for j in compounds_remaining
				old_phase = compounds[j]
				if	((old_phase == assets[COMPOUND_TYPES[j]].trans_prob_col.APP) || (!invested[j]))
					time_in_phase[j]+=0*1
					continue				
				end
				perform_transitions_ffn(j,time_required[j,compounds[j]],success[j,compounds[j]])
                transitioned[j]= compounds[j]!=old_phase
				if (compounds[j] == sell_phase[j]) && (assets[COMPOUND_TYPES[j]].sale_delay[compounds[j]]<=0)
					sell_compound_ffn(j,i)
					delete!(compounds_remaining,j)
                elseif ((compounds[j]>=sell_phase[j] && (assets[COMPOUND_TYPES[j]].sale_delay[compounds[j]]>0) && time_since_target[j]>=assets[COMPOUND_TYPES[j]].sale_delay[compounds[j]] ) 
					|| compounds[j]==assets[COMPOUND_TYPES[j]].trans_prob_col.APP)
					sell_compound_ffn(j,i)
					delete!(compounds_remaining,j)
				elseif compounds[j]==assets[COMPOUND_TYPES[j]].trans_prob_col.DSC 
					if debug > 1
						println("Withdrawing $j from $old_phase.")
					end
					withdrawals[nsimu,old_phase]+=1
					withdrawal_times[nsimu,i]+=1
					time_in_phase[j]=0	
					delete!(compounds_remaining,j)
				elseif transitioned[j]
					transition_compound_ffn(j)			
				else
					time_in_phase[j]+=1
                    if compounds[j]>=sell_phase[j] || compounds[j]==assets[COMPOUND_TYPES[j]].trans_prob_col.APP
                        time_since_target[j]+=1
                    end
				end
                transitioned[j]=false
			end

            current_cash  = cash[i]-simu.fixedcost/2
			current_guarantee  = guarantee[i]

			# ----------- accounting balance
			A1_realized_vals[nsimu,i] = bond_value[1]
			A2_realized_vals[nsimu,i] = bond_value[2]
			
			#------------ next if not yet in default, check for default
			if !in_default 
              in_default = check_for_default_ffn(i,current_cash,current_guarantee)
			end #if ~in_default
			
			#------------ next if not in default pay P&I if in default liquidate portfolio
			if !in_default
               # 	Pay service fees
                current_cash = pay_servicing_ffn(i,current_cash,current_guarantee)
				#Pay P&I
                current_cash = pay_interest_ffn(i,current_cash,current_guarantee)
                current_cash = pay_principal_ffn(i,current_cash,current_guarantee)
			if(current_cash<0)
			    	current_cash , current_guarantee = draw_from_guarantee_ffn(i,current_cash,current_guarantee)
			end	
		
				#------------ next check coverage
				for b = 1:NBONDS #calc IC coverage
					if bond_value[b]>0		
						cash_for_coverage=current_cash+sum(cash[i+(1:bonds.IC_pers)])
                        ic_ratio[b,i], cost_due, K = ic_test_result_ffn(b,i,cash_for_coverage)  
						ic_shortfall[b,i] = (bonds.interest_coverage[b]*cost_due) - K
					else 
                        ic_shortfall[b] = 0
                    end #if bond_value(b)>0
				end #for b = 1: NBONDS

				#------------ check equity svc coverage
				eq_svc_due = unpaid_svc
                ##TODO: FIX ASSET TYPES WITH DIFFERENT MAX SALE TIMES
				for k = 1:2#ceil(maximum(assets[1].sale_time))
					eq_svc_due = eq_svc_due + bond_value[length(bond_value)]*bonds.servicing_rate
				end
				eq_shortfall =  eq_svc_due - current_cash
				
				#------------ next if IC too low, sell compounds to cover
				max_shortfall = maximum([ic_shortfall[:,i],eq_shortfall]) 
				if max_shortfall > 0 
					#Mutates cash,compounds.
					if debug>0
						println("Shortfall found $max_shortfall.")
						println("Equity part: $eq_shortfall.")
						println("Current cash: $current_cash.")
					end
					compounds_remaining = sell_compounds_to_cover_shortfall_ffn(compounds_remaining,max_shortfall,i)
					OK_to_fund = false
				else 
					OK_to_fund = true
				end
				#------------ next if IC is OK fund next phases of research for compounds 
				if OK_to_fund 
					if i<NPERS 
						next_pers = (i+1):min(NPERS,i+bonds.IC_pers)
						cash_required_for_future=simu.fixedcost/2*bonds.IC_pers
						for per in next_pers, b in 1:NBONDS
							cash_required_for_future+=AMORT_SCHED[b,per]+bonds.coupon[b]*bond_value[b]+bonds.servicing_rate*bond_value[b]
						end
						cash_required_for_future+=bonds.servicing_rate*bond_value[end]
						cash_to_invest           = max(current_cash-cash_required_for_future,0)
							
						for k in compounds_remaining
		               		if (compounds[k]==assets[COMPOUND_TYPES[k]].trans_prob_col.APP ||  invested[k])  
								continue
                            end
							compounds_to_fund[nsimu,i]+=1
						end
						if cash_to_invest>0 
							cash_spent             = 0
                            sold_index = IntSet()
							for k in compounds_remaining
								if (compounds[k]==assets[COMPOUND_TYPES[k]].trans_prob_col.APP ||	invested[k])  
									continue
								end
								if invest_cost[k]  <= cash_to_invest #&& iceil(mean_time_required[k,compounds[k]]) < (NPERS-i)
									invested[k]  	= true
									cash_to_invest -= invest_cost[k]
									cash_spent     += invest_cost[k]

                                    total_invest_cost[k]+=invest_cost[k] ## FOR PERFORMANCE FEE TRACKING

									compounds_funded[nsimu,i] +=1
                                elseif invest_cost[k] <= cash_to_invest
                                    #println("SKIPPING FUNDING SLOW TRIAL in period $i with duration $(assets[COMPOUND_TYPES[k]].dur[compounds[k]])")
                                    sell_compound_ffn(k,i)
                                    push!(sold_index,k)
								end
							end
                            setdiff!(compounds_remaining,sold_index)

							current_cash -= cash_spent 
							cash_for_investment[nsimu,i] += cash_spent

                            ## NEW PARAMETER FILES NEED THIS
                            for ctr = 1:NCOMPOUNDS
                               if (STARTING_PERIOD[ctr] < i && compounds[ctr] > 0)
						           future_cost_est = sell_phase[ctr]<=0 ? 0 : assets[COMPOUND_TYPES[ctr]].budget[compounds[ctr]]
                                   cash_to_invest = cash_to_invest - future_cost_est
                               end
                            end
                            
                            # number of compounds we actually buy per phase LAST PERIOD
                            num_compounds      =  zeros(NSTATES)
							cash_spent = 0
						    for ctr = 1:NCOMPOUNDS
						        if STARTING_PERIOD[ctr]==i
						            price_tmp = invest_cost[ctr] + assets[COMPOUND_TYPES[ctr]].upfront[compounds[ctr]]
						            future_cost_est = sell_phase[ctr]<=0 ? 0 : assets[COMPOUND_TYPES[ctr]].budget[compounds[ctr]]
						            if cash_to_invest   >= price_tmp + future_cost_est
						                money_spent[ctr]     =  price_tmp
						                money_saved[ctr]     =  future_cost_est
						                cash_to_invest  -= price_tmp + future_cost_est
									    cash_spent  += price_tmp
						                num_compounds[compounds[ctr]] +=  1
						                push!(compounds_remaining,ctr)
						                if debug > 1
					    	                println("buying: $ctr\t for $price_tmp, saving $future_cost_est with cash left: $cash_to_invest")
					        	        end
					            	else
						                compounds[ctr] = assets[COMPOUND_TYPES[ctr]].trans_prob_col.DSC
						            end
						        end
						    end

                            compounds_bought[nsimu,i,:] = num_compounds
                            cash_for_buying[nsimu,i] += cash_spent
                            
                            current_cash -= cash_spent 

                            ##Subtract future purchases from budget.
                            for ctr = 1:NCOMPOUNDS
                                if STARTING_PERIOD[ctr]>i
                                    price_tmp = invest_cost[ctr] + assets[COMPOUND_TYPES[ctr]].upfront[compounds[ctr]]
                                    future_cost_est = sell_phase[ctr]<=0 ? 0 : assets[COMPOUND_TYPES[ctr]].budget[compounds[ctr]]
                                    if cash_to_invest   >= price_tmp + future_cost_est
                                        cash_to_invest  -= price_tmp + future_cost_est
                                    end
                                end
                            end
                            
                            if i>simu.MINPAYOFFTIME

                    			payoff_slice = slice(payoff,nsimu,i,:)
                                payoff_slice[:] += max(cash_to_invest,0)*params.simu.tranche_cashweights[:]
                                current_cash -= max(cash_to_invest,0)
                                cash_to_invest=0
                            end
                            
						end #if cash to invest > 0
                        if current_cash < 0
                            error("Cash is negative in period $i")
                        end
                    end #if(i<NPERS)
                end #if(OK_to_fund)
			else # already in default
                if debug > 0 
					println("Early liquidation:\n $compounds_remaining")
				end
				compounds_remaining = liquidate_portfolio_ffn(compounds_remaining,i)
				current_cash = pay_servicing_ffn(i,current_cash, current_guarantee)  # need to check to make sure missed payements are tracked

				#PAY P&I
				for b = 1:NBONDS # pay down bonds sequentially
                    if current_cash+current_guarantee>0
                        I  = min(current_cash+current_guarantee,bonds.coupon[b]*bond_value[b])  
                        interest_paid[b,i]  = I
                        current_cash       -= I
                        
                        P                   = min(current_cash+current_guarantee,bond_value[b])
                        principal_paid[b,i] = P
                        bond_value[b]      -= P
                        if bond_value[b]<0.0001
                            bond_value[b] = 0
                        end
                        current_cash -= P
					end
                end
				if current_cash<0
               		current_cash, current_guarantee = draw_from_guarantee_ffn(i,current_cash, current_guarantee)
					if i > 11
						println("Drawing in default in period $i")
					end
                end
				
            end #else ~in_default

			cash[i] = current_cash
			guarantee[i] = current_guarantee
            
			if i!=NPERS 
				#cash left over at end of period acquires interest
				cash[i+1]             = cash[i+1]+current_cash*(1+bonds.cash_accrual_int_rate)
				cash_begin_per[nsimu,i+1] = cash[i+1]
                
				guarantee[i+1]        = sum(bond_value[1:NBONDS]) > 0 ? guarantee[i] : 0
                guarantee_begin_per[nsimu,i+1] = guarantee[i+1]
			end
		end #for i = 2 : NPERS	%%END TIMESTEP ITERATION%%%%%
         
		for j=1:NCOMPOUNDS
			if compounds[j] > 0 
				final_compounds[compounds[j]]+=1
			end
		end
     	if debug > 0 
			println("Liquidation:\n $compounds_remaining")
		end
		compounds_remaining = liquidate_portfolio_ffn(compounds_remaining,NPERS)
		#Experimental, untested repayment parameter.
        if ce.ce_repay
           current_cash, current_guarantee = repay_guarantee_ffn(NPERS,current_cash, current_guarantee)
        end
        for i = NPERS:(HORIZON-1) 
			if debug > 0
				println("Wrapping up, period $i")
			end
			current_cash = cash[i]
			if i <LIQHORIZON
                current_cash  = pay_servicing_ffn(i,current_cash, current_guarantee)
    			#Experimental, untested repayment parameter.
    			if ce.ce_repay
                    current_cash, current_guarantee = repay_guarantee_ffn(i,current_cash, current_guarantee)
                end
    			if current_cash<0
           			current_cash, current_guarantee = draw_from_guarantee_ffn(i,current_cash, current_guarantee)
            	end
            end

            if i>simu.MINPAYOFFTIME
				payoff_slice = slice(payoff,nsimu,i,:)
				payoff_slice[:] += max(current_cash,0)*params.simu.tranche_cashweights[:]
                current_cash = 0
            end
            cash[i]=current_cash
			guarantee[i]=current_guarantee
			cash[i+1]   += cash[i] # last sales; need to add check for NPERS>=HORIZON
			guarantee[i+1] = guarantee[i]
			cash_begin_per[nsimu,i+1] = cash[i+1]
        end
				
		cash_realized[nsimu,:]    = cash
		
		A1_paid[nsimu,:]   = principal_paid[1,:]+interest_paid[1,:]
		A2_paid[nsimu,:]   = principal_paid[2,:]+interest_paid[2,:]
        for ne = 1:NEQUITY
            return_on_equity[nsimu,ne] = max((cash[LIQHORIZON]*tranche_cashweights[ne]+sum(payoff[nsimu,1:LIQHORIZON,ne]+reserve_cash[nsimu,1:LIQHORIZON,ne]) - bonds.nominal[NBONDS+1]*params.simu.tranche_cashweights[ne])./(bonds.nominal[NBONDS+1]*params.simu.tranche_cashweights[ne]),-1)
            weight_cash[nsimu,ne] = tranche_cashweights[ne]
            weight_tranche[nsimu,:,ne] =tranche_weights[:,ne]
        end

 
    end #for s = 1:NSIMUS 	###END SIM COUNT LOOP#####
    return Results(cash_begin_per,
		   			cash_realized,
                    payoff,
					reserve,
					reserve_cash,
                    weight_tranche,
                    weight_cash,
				    cash_for_investment,
					cash_for_buying,
				    A1_realized_vals,
				    A2_realized_vals,
				    AMORT_SCHED,
				    A1_paid,
				    A2_paid,
				    IC_ratio,
				    return_on_equity,
                    irr_on_equity,
				    sales,
				    withdrawals,
				    sale_times,
				    withdrawal_times,
				    extra_cash_from_coverage,
                    cash_from_milestones,
				    history_service,
				    compounds_to_fund,
				    compounds_funded,
				    squeeze(mean(compounds_bought,1),1),
                    squeeze(mean(compounds_current,1),1),
				    params,
				    sell_value,
				    final_compounds,
				    A1_defaults,
				    A2_defaults,
				    svc_defaults,
					guarantee_begin_per,
				    guarantee_draw,
				    drew_on_guarantee)
end
        

end
#COPYRIGHT 2012,2013
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#


