using DrugParams
using ParamUtilitiesOriginal
using Distributions

#This function initializes all parameters used by the simulation code
#The function contains all default values used in the code
#Input:
#Output:
#@params is of type struct and has 3 fields simu (simulation parameters), assets (asset parameters), bonds (bond parameters)
function init_params_fn()
	PHASES=["DSC","PRE","P1","P2","P3","NDA","APP"]
    PARAMETER=["vmu","vsigma","vmx","upfront","milestone","mu","sigma","mx","futurecostest"]
    # ASSET PARAMETERS
    rho = 0.2


    pricing_params  =  [ 
    #vmu     			vsigma   vmx    UpFront   			Milestone    		mu       			sigma      max		FutureCostEst
    0      			1         0  	0		          	0     				0       			0          0      0;         #DSC
    2.354733621857371 0.939   100  	2.506648325603443 	1.253324162801722  1.573797446769595  0.785050619780075      20    	13;   #PRE
    2.976737648081240 0.939   250  	7.519944976810328 	3.759972488405164  2.722501027059631  0.732114106934504    50    	0;   #P1
    3.996945974057142 0.939   500   20.053186604827545	10.026593302413772 3.702027419856847  0.795995957236353     	120   	0;   #P2
    5.802748472629424 0.939  1000  	75.199449768103278	37.599724884051639 5.065276957812039	0.633123896262475   	500    0;    #P3
    7.277762431143847 0.939  2500  	0          		0    				0       			0          0      0;    #NDA
    7.239151725642049 0.939  5000 	0          		0    				0       			0          0      0;  #APP
    ];

    vmu=pricing_params[:,1]
    vsigma=pricing_params[:,2]
    vmx=pricing_params[:,3]
    upfront=pricing_params[:,4]
    milestone=pricing_params[:,5]
    cmu=pricing_params[:,6]
    csigma=pricing_params[:,7]
    cmx=pricing_params[:,8]
    budget=pricing_params[:,9]
    
    Distribution = "CLN"

    valueDist = Array(UnivariateDistribution,length(PHASES))
    costDist = Array(UnivariateDistribution,length(PHASES))
    for i = 1:length(PHASES)
        ##Convert from LN using first,second moments
        valueDist[i]=convertLNtoGeneralized(vmu[i],vsigma[i],vmx[i],Distribution)
        costDist[i]=convertLNtoGeneralized(cmu[i],csigma[i],cmx[i],Distribution)
    end

    
	trans_prob = [ 
	#DSC           		PRE  P1       			P2 			        P3 			        NDA 		       APP 
    1.000000000000000  0   0        			0         			0.000000000000000 	0         			0; #DSC
    0.155000000000000  0.5 0.345    			0         			0.000000000000000	0         			0; #PRE
    0.053215560704185  0   0.807859886086702	0.133437226118754 	0.005307429700172	0.000157443915296	0.000022453474890; #P1
    0.085093613975923  0   0    				0.844734609586595	0.066750087871169	0.002866145785922 	0.000555542780392; #P2
    0.062545675724766  0   0        			0         			0.848420924719526	0.068234113621235 	0.020799285934472; #P3
    0.021663504081486  0   0        			0         			0         			0.566729918370271 	0.411606577548242; #NDA
    0			        0   0        			0         			0         			0         			1.000000000000000; #APP 
    ];
    
    trans_prob_row = StateDict(1,2,3,4,5,6,7,-1)
    trans_prob_col = trans_prob_row
    
    timeMode = "Markov" ##markov,period, or years (default)
    timeDist = Array(UnivariateDistribution,length(PHASES))
    
    target = 4
    sell_in_phase = [-1,target,target,target,target,target,target]
	sale_delay = zeros(7)
    sale_time = 1.5*ones(7)
    ratio_unchanged = 1
    equity_stake    = 0.85
    nPhases = length(PHASES)
    assets = Assets(rho,trans_prob,trans_prob_col,
                    upfront,  milestone,
                    valueDist, 
                    [],
                    costDist, 
                    ones(nPhases), ##No structured exits
                    StrucExit(nPhases),
                    budget,
                    sell_in_phase,
					sale_delay,
                    sale_time,
                    ratio_unchanged,
                    equity_stake,
                    timeMode,
                    timeDist,
                    [],
                    1.0, ##no scientific minimum success
                    0 ##no scientific correlation
                    )
   

    #CE PARAMETERS
    ce_amt	   = 0.0
    ce_repay   = false
    ce_premium = 0
    ce         =Ce(ce_amt,ce_repay,ce_premium)
    
	#BOND PARAMETERS
    capital_structure = [0.25, 0.15, 0.6] #A1, A2, EQ
    IC_pers=2
	interest_coverage = [1.75, 3.5, 0]  #A1, A2, EQ
    amort_timing  = reshape([5,9,8,12],2,2)
    
    coupon = [0.05/2, 0.08/2, 0]
    servicing_rate = 0.0025
    svc_accrual_int_rate = 0.07/2
    cash_accrual_int_rate = 0.01/2

    initial_cash= 5000
	nominal = initial_cash*capital_structure
    bonds=Bonds(capital_structure,IC_pers,interest_coverage,amort_timing,coupon,servicing_rate,
				svc_accrual_int_rate,cash_accrual_int_rate,nominal)
	
	# SIMU PARAMETERS
    NSIMUS     	= 100000
    TIMESTEPS	=  13
    STEPS_YR    = 2
	
    #Time,phases,asset types
	initial_compounds      = zeros(TIMESTEPS,length(PHASES),1)
    initial_compounds[1,:,:] = [0, 100, 100, 0, 0, 0, 0]

 	reserve_flag = false
    fixedcost = 0
    MINPAYOFFTIME       = typemax(Int64)
    tranche_weights = ones(int(sum(vec(initial_compounds))),1)
    tranche_cashweights = [1.0]
    tranche_capratio = [Inf]
    performance_fee=NaN
	simu=Simu(NSIMUS,TIMESTEPS,STEPS_YR,initial_cash,initial_compounds,reserve_flag,
        tranche_weights,tranche_cashweights,tranche_capratio,fixedcost,MINPAYOFFTIME,
        performance_fee,[],[],TIMESTEPS)
	
    #create global parameter list
    return Params(simu,[assets], bonds, ce)
end
#COPYRIGHT 2012
# This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    